# Generated by Django 4.1.5 on 2023-01-19 12:11

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("email_log", "0002_email"),
    ]

    operations = [
        migrations.AddField(
            model_name="email",
            name="html_message",
            field=models.TextField(blank=True, verbose_name="HTML message"),
        ),
        migrations.CreateModel(
            name="Attachment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("file", models.FileField(upload_to="", verbose_name="file")),
                (
                    "name",
                    models.CharField(
                        help_text="filename", max_length=255, verbose_name="name"
                    ),
                ),
                ("mimetype", models.CharField(blank=True, default="", max_length=255)),
                (
                    "email",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="attachments",
                        to="email_log.email",
                        verbose_name="email",
                    ),
                ),
            ],
            options={
                "verbose_name": "attachment",
                "verbose_name_plural": "attachments",
            },
        ),
    ]
